@@ -15,8 +15,6 @@
  */
 package org.kie.hacep.core.infra.consumer;
 
-import java.time.Duration;
-import java.time.temporal.ChronoUnit;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -35,13 +33,12 @@ import org.apache.kafka.common.errors.WakeupException;
 import org.kie.hacep.Config;
 import org.kie.hacep.EnvConfig;
 import org.kie.hacep.consumer.DroolsConsumerHandler;
-import org.kie.hacep.core.Bootstrap;
-import org.kie.hacep.core.infra.DeafultSessionSnapShooter;
+import org.kie.hacep.core.infra.DefaultSessionSnapShooter;
 import org.kie.hacep.core.infra.OffsetManager;
 import org.kie.hacep.core.infra.SnapshotInfos;
 import org.kie.hacep.core.infra.election.State;
 import org.kie.hacep.core.infra.utils.ConsumerUtils;
-import org.kie.hacep.message.ControlMessage;
+import org.kie.remote.message.ControlMessage;
 import org.kie.hacep.util.Printer;
 import org.kie.hacep.util.PrinterUtil;
 import org.kie.remote.DroolsExecutor;
@@ -49,7 +46,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import static org.kie.remote.util.SerializationUtil.deserialize;
-
 /**
  * The default consumer relies on the Consumer thread and
  * is based on the loop around poll method.
@@ -58,50 +54,53 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
 
     private Logger logger = LoggerFactory.getLogger(DefaultKafkaConsumer.class);
     private Map<TopicPartition, OffsetAndMetadata> offsetsEvents = new HashMap<>();
-    private org.apache.kafka.clients.consumer.Consumer<String, T> kafkaConsumer, kafkaSecondaryConsumer;
+    private Consumer<String, T> kafkaConsumer, kafkaSecondaryConsumer;
     private DroolsConsumerHandler consumerHandler;
-    private volatile State currentState;
     private volatile String processingKey = "";
-    private volatile long processingKeyOffset;
-    private volatile boolean leader = false;
-    private volatile boolean started = false;
-    private volatile boolean processingLeader, processingReplica = false;
-    private volatile boolean pollingEvents, pollingControl = true;
+    private volatile long processingKeyOffset, lastProcessedControlOffset, lastProcessedEventOffset;
+    private volatile boolean started, exit = false;
+    private volatile State currentState = State.REPLICA;
+    private volatile PolledTopic polledTopic = PolledTopic.CONTROL;
     private int iterationBetweenSnapshot;
     private List<ConsumerRecord<String, T>> eventsBuffer;
     private List<ConsumerRecord<String, T>> controlBuffer;
-    private AtomicInteger counter = new AtomicInteger(0);
+    private AtomicInteger counter ;
     private SnapshotInfos snapshotInfos;
-    private DeafultSessionSnapShooter snapShooter;
+    private DefaultSessionSnapShooter snapShooter;
     private Printer printer;
-    private EnvConfig config;
+    private EnvConfig envConfig;
     private Logger loggerForTest;
     private volatile boolean askedSnapshotOnDemand;
 
     public DefaultKafkaConsumer(EnvConfig config) {
-        this.config = config;
-        iterationBetweenSnapshot = config.getIterationBetweenSnapshot();
-        this.printer = PrinterUtil.getPrinter(config);
-        if (config.isUnderTest()) {
-            loggerForTest = PrinterUtil.getKafkaLoggerForTest(config);
+        this.envConfig = config;
+        if(this.envConfig.isSkipOnDemandSnapshot()){
+            counter = new AtomicInteger(0);
+        }
+        iterationBetweenSnapshot = this.envConfig.getIterationBetweenSnapshot();
+        this.printer = PrinterUtil.getPrinter(this.envConfig);
+        if (this.envConfig.isUnderTest()) {
+            loggerForTest = PrinterUtil.getKafkaLoggerForTest(this.envConfig);
         }
     }
 
     public void initConsumer(ConsumerHandler consumerHandler) {
         this.consumerHandler = (DroolsConsumerHandler) consumerHandler;
-        this.snapShooter = this.consumerHandler.getSnapshooter();
+        this.snapShooter = this.consumerHandler.getSessionSnapShooter();
         this.kafkaConsumer = new KafkaConsumer<>(Config.getConsumerConfig("PrimaryConsumer"));
-        if (!leader) {
+        if (currentState.equals(State.REPLICA)) {
             this.kafkaSecondaryConsumer = new KafkaConsumer<>(Config.getConsumerConfig("SecondaryConsumer"));
         }
     }
 
-    private void restartConsumer() {
-        logger.info("Restart Consumers");
-        snapshotInfos = snapShooter.deserialize();
+    protected void restartConsumer() {
+        if (logger.isInfoEnabled()) {
+            logger.info("Restart Consumers");
+        }
+        snapshotInfos = snapShooter.deserialize();//is still useful ?
         kafkaConsumer = new KafkaConsumer<>(Config.getConsumerConfig("PrimaryConsumer"));
         assign();
-        if (!leader) {
+        if (currentState.equals(State.REPLICA)) {
             kafkaSecondaryConsumer = new KafkaConsumer<>(Config.getConsumerConfig("SecondaryConsumer"));
         } else {
             kafkaSecondaryConsumer = null;
@@ -111,70 +110,78 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
     @Override
     public void stop() {
         stopConsume();
-        stopPollingControl();
-        stopPollingEvents();
         kafkaConsumer.wakeup();
         if (kafkaSecondaryConsumer != null) {
             kafkaSecondaryConsumer.wakeup();
         }
+        exit = true;
         consumerHandler.stop();
     }
 
     @Override
     public void updateStatus(State state) {
-        if (started) {
+        boolean changedState = !state.equals(currentState);
+        if(currentState == null ||  changedState){
+            currentState = state;
+        }
+        if (started && changedState && !currentState.equals(State.BECOMING_LEADER)) {
             updateOnRunningConsumer(state);
-        } else {
+        } else if(!started) {
             if (state.equals(State.REPLICA)) {
                 //ask and wait a snapshot before start
-                if (!config.isSkipOnDemanSnapshot() && !askedSnapshotOnDemand) {
+                if (!envConfig.isSkipOnDemandSnapshot() && !askedSnapshotOnDemand) {
+                    if (logger.isInfoEnabled()) {
+                        logger.info("askAndProcessSnapshotOnDemand:");
+                    }
                     askAndProcessSnapshotOnDemand();
                 }
             }
             //State.BECOMING_LEADER won't start the pod
             if (state.equals(State.LEADER) || state.equals(State.REPLICA)) {
+                if (logger.isInfoEnabled()) {
+                    logger.info("enableConsumeAndStartLoop:{}", state);
+                }
                 enableConsumeAndStartLoop(state);
             }
         }
-        currentState = state;
     }
 
-    private void askAndProcessSnapshotOnDemand() {
+    protected void askAndProcessSnapshotOnDemand() {
         askedSnapshotOnDemand = true;
-        boolean completed = consumerHandler.initializeKieSessionFromSnapshotOnDemand(config);
+        boolean completed = consumerHandler.initializeKieSessionFromSnapshotOnDemand(envConfig);
+        if (logger.isInfoEnabled()) {
+            logger.info("askAndProcessSnapshotOnDemand:{}", completed);
+        }
         if (!completed) {
             throw new RuntimeException("Can't obtain a snapshot on demand");
         }
     }
 
-    private void assign() {
-        if (leader) {
+    protected void assign() {
+        if (currentState.equals(State.LEADER)) {
             assignAsALeader();
         } else {
-            assignNotLeader();
+            assignReplica();
         }
     }
 
-    private void assignAsALeader() {
-        assignConsumer(kafkaConsumer,
-                       config.getEventsTopicName());
+    protected void assignAsALeader() {
+        assignConsumer(kafkaConsumer, envConfig.getEventsTopicName());
     }
 
-    private void assignNotLeader() {
-        assignConsumer(kafkaConsumer,
-                       config.getEventsTopicName());
-        assignConsumer(kafkaSecondaryConsumer,
-                       config.getControlTopicName());
+    protected void assignReplica() {
+        assignConsumer(kafkaConsumer, envConfig.getEventsTopicName());
+        assignConsumer(kafkaSecondaryConsumer, envConfig.getControlTopicName());
     }
 
-    private void assignConsumer(Consumer<String, T> kafkaConsumer, String topic) {
+    protected void assignConsumer(Consumer<String, T> kafkaConsumer, String topic) {
 
         List<PartitionInfo> partitionsInfo = kafkaConsumer.partitionsFor(topic);
         Collection<TopicPartition> partitionCollection = new ArrayList<>();
 
         if (partitionsInfo != null) {
             for (PartitionInfo partition : partitionsInfo) {
-                    partitionCollection.add(new TopicPartition(partition.topic(), partition.partition()));
+                partitionCollection.add(new TopicPartition(partition.topic(), partition.partition()));
             }
 
             if (!partitionCollection.isEmpty()) {
@@ -189,11 +196,17 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
             kafkaConsumer.assignment().forEach(topicPartition -> kafkaConsumer.seek(partitionCollection.iterator().next(),
                                                                                     snapshotInfos.getOffsetDuringSnapshot()));
         } else {
-            kafkaConsumer.assignment().forEach(topicPartition -> kafkaConsumer.seekToBeginning(partitionCollection));
+            if(currentState.equals(State.LEADER)){
+                kafkaConsumer.assignment().forEach(topicPartition -> kafkaConsumer.seek(partitionCollection.iterator().next(),
+                                                                                        lastProcessedEventOffset));
+            }else if(currentState.equals(State.REPLICA)){
+                kafkaConsumer.assignment().forEach(topicPartition -> kafkaConsumer.seek(partitionCollection.iterator().next(),
+                                                                                        lastProcessedControlOffset));
+            }
         }
     }
 
-    public void poll(int durationMillis) {
+    public void poll() {
 
         final Thread mainThread = Thread.currentThread();
         Runtime.getRuntime().addShutdownHook(new Thread(() -> {
@@ -219,8 +232,8 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
         }
 
         try {
-            while (true) {
-                consume(durationMillis);
+            while (!exit) {
+                consume();
             }
         } catch (WakeupException e) {
             //nothind to do
@@ -246,171 +259,156 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
         }
     }
 
-    private void updateOnRunningConsumer(State state) {
-        if (state.equals(State.LEADER) && !leader) {
+    protected void updateOnRunningConsumer(State state) {
+        logger.info("updateOnRunning COnsumer");
+        if (state.equals(State.LEADER) ) {
             DroolsExecutor.setAsLeader();
             restart(state);
-        } else if (state.equals(State.REPLICA) && leader) {
+        } else if (state.equals(State.REPLICA)) {
             DroolsExecutor.setAsReplica();
             restart(state);
         }
     }
 
-    private void restart(State state) {
+    protected void restart(State state) {
         stopConsume();
         restartConsumer();
         enableConsumeAndStartLoop(state);
     }
 
-    private void enableConsumeAndStartLoop(State state) {
-        if (state.equals(State.LEADER) && !leader) {
-            leader = true;
+    protected void enableConsumeAndStartLoop(State state) {
+        if (state.equals(State.LEADER)) {
+            currentState = State.LEADER;
             DroolsExecutor.setAsLeader();
-            stopLeaderProcessing();// we starts to processing only when the last key readed on bootstrap is reached
-        } else if (state.equals(State.REPLICA) && leader) {
-            leader = false;
+
+        } else if (state.equals(State.REPLICA) ) {
+            currentState = State.REPLICA;
             kafkaSecondaryConsumer = new KafkaConsumer<>(Config.getConsumerConfig("SecondaryConsumer"));
             DroolsExecutor.setAsReplica();
-            startProcessingNotLeader();
-            startPollingEvents();
-            stopPollingControl();
-        } else if (state.equals(State.REPLICA) && !leader) {
-            leader = false;
-            kafkaSecondaryConsumer = new KafkaConsumer<>(Config.getConsumerConfig("SecondaryConsumer"));
-            DroolsExecutor.setAsReplica();
-            startProcessingNotLeader();
-            stopPollingEvents();
-            startPollingControl();
         }
         setLastProcessedKey();
         assignAndStartConsume();
     }
 
-    private void setLastProcessedKey() {
-        ControlMessage lastControlMessage = ConsumerUtils.getLastEvent(config.getControlTopicName(),
-                                                                       config.getPollTimeout());
+    protected void setLastProcessedKey() {
+        ControlMessage lastControlMessage = ConsumerUtils.getLastEvent(envConfig.getControlTopicName(), envConfig.getPollTimeout());
         settingsOnAEmptyControlTopic(lastControlMessage);
         processingKey = lastControlMessage.getId();
         processingKeyOffset = lastControlMessage.getOffset();
     }
 
-    private void settingsOnAEmptyControlTopic(ControlMessage lastWrapper) {
+    protected void settingsOnAEmptyControlTopic(ControlMessage lastWrapper) {
         if (lastWrapper.getId() == null) {// completely empty or restart of ephemeral already used
-            if (leader) {
-                startProcessingLeader();
-            } else {
-                stopProcessingNotLeader();
-                stopPollingEvents();
-                startPollingControl();// the nodes start to poll only the controlTopic until a event is available
+            if (currentState.equals(State.REPLICA)) {
+                pollControl();
             }
         }
     }
 
-    private void assignAndStartConsume() {
+    protected void assignAndStartConsume() {
         assign();
         startConsume();
     }
 
-    private void consume(int millisTimeout) {
+    protected void consume() {
         if (started) {
-            if (leader) {
-                defaultProcessAsLeader(millisTimeout);
+            if (currentState.equals(State.LEADER)) {
+                defaultProcessAsLeader();
             } else {
-                defaultProcessAsAReplica(millisTimeout);
+                defaultProcessAsAReplica();
             }
         }
     }
 
-    private void defaultProcessAsLeader(int millisTimeout) {
-        startPollingEvents();
-        startProcessingLeader();
-        ConsumerRecords<String, T> records = kafkaConsumer.poll(Duration.of(millisTimeout,
-                                                                            ChronoUnit.MILLIS));
-        for (ConsumerRecord<String, T> record : records) {
-            processLeader(record,
-                          counter);
+    protected void defaultProcessAsLeader() {
+        pollEvents();
+        if (eventsBuffer != null && eventsBuffer.size() > 0) { // events previously readed and not processed
+            consumeEventsFromBufferAsALeader();
+        }
+        ConsumerRecords<String, T> records = kafkaConsumer.poll(envConfig.getPollDuration());
+        if (!records.isEmpty() && eventsBuffer == null) {
+            ConsumerRecord<String, T> first = records.iterator().next();
+            eventsBuffer = records.records(new TopicPartition(first.topic(), first.partition()));
+            consumeEventsFromBufferAsALeader();
+        } else {
+            pollControl();
         }
     }
 
-    private void processLeader(ConsumerRecord<String, T> record,
-                               AtomicInteger counter) {
-        if (processingLeader) {
-            if (config.isSkipOnDemanSnapshot()) {
-                handleSnapshotBetweenIteration(record,
-                                               counter);
-            } else {
-                consumerHandler.process(ItemToProcess.getItemToProcess(record),
-                                        currentState);
-            }
-            processingKey = record.key();// the new processed became the new processingKey
-            saveOffset(record,
-                       kafkaConsumer);
+    protected void processLeader(ConsumerRecord<String, T> record) {
+
+        if (envConfig.isSkipOnDemandSnapshot()) {
+            handleSnapshotBetweenIteration(record);
+        } else {
+            consumerHandler.process(ItemToProcess.getItemToProcess(record), currentState);
         }
-        if (logger.isInfoEnabled() || config.isUnderTest()) {
-            printer.prettyPrinter("DefaulKafkaConsumer.processLeader record:{}",
-                                  record,
-                                  processingLeader);
+        processingKey = record.key();// the new processed became the new processingKey
+        saveOffset(record, kafkaConsumer);
+
+        if (logger.isInfoEnabled() || envConfig.isUnderTest()) {
+            printer.prettyPrinter("DefaulImprovedKafkaConsumer.processLeader record:{}", record, true);
         }
     }
 
-    private void handleSnapshotBetweenIteration(ConsumerRecord<String, T> record,
-                                                AtomicInteger counter) {
+    protected void consumeEventsFromBufferAsALeader() {
+        for (ConsumerRecord<String, T> record : eventsBuffer) {
+            processLeader(record);
+        }
+        eventsBuffer = null;
+    }
+
+    protected void handleSnapshotBetweenIteration(ConsumerRecord<String, T> record) {
         int iteration = counter.incrementAndGet();
         if (iteration == iterationBetweenSnapshot) {
             counter.set(0);
-            consumerHandler.processWithSnapshot(ItemToProcess.getItemToProcess(record),
-                                                currentState);
+            consumerHandler.processWithSnapshot(ItemToProcess.getItemToProcess(record), currentState);
         } else {
-            consumerHandler.process(ItemToProcess.getItemToProcess(record),
-                                    currentState);
+            consumerHandler.process(ItemToProcess.getItemToProcess(record), currentState);
         }
     }
 
-    private void defaultProcessAsAReplica(int millisTimeout) {
-        if (pollingEvents) {
+    protected void defaultProcessAsAReplica() {
+        if (polledTopic.equals(PolledTopic.EVENTS)) {
             if (eventsBuffer != null && eventsBuffer.size() > 0) { // events previously readed and not processed
                 consumeEventsFromBufferAsAReplica();
             }
-            ConsumerRecords<String, T> records = kafkaConsumer.poll(Duration.of(millisTimeout,
-                                                                                ChronoUnit.MILLIS));
-            if (records.count() > 0) {
+            ConsumerRecords<String, T> records = kafkaConsumer.poll(envConfig.getPollDuration());
+            if (!records.isEmpty()) {
                 ConsumerRecord<String, T> first = records.iterator().next();
-                eventsBuffer = records.records(new TopicPartition(first.topic(),
-                                                                  first.partition()));
+                eventsBuffer = records.records(new TopicPartition(first.topic(), first.partition()));
                 consumeEventsFromBufferAsAReplica();
             } else {
-                stopPollingEvents();
-                startPollingControl();
+                pollControl();
             }
         }
 
-        if (pollingControl) {
+        if (polledTopic.equals(PolledTopic.CONTROL)) {
 
             if (controlBuffer != null && controlBuffer.size() > 0) {
                 consumeControlFromBufferAsAReplica();
             }
 
-            ConsumerRecords<String, T> records = kafkaSecondaryConsumer.poll(Duration.of(millisTimeout,
-                                                                                         ChronoUnit.MILLIS));
+            ConsumerRecords<String, T> records = kafkaSecondaryConsumer.poll(envConfig.getPollDuration());
             if (records.count() > 0) {
                 ConsumerRecord<String, T> first = records.iterator().next();
-                controlBuffer = records.records(new TopicPartition(first.topic(),
-                                                                   first.partition()));
+                controlBuffer = records.records(new TopicPartition(first.topic(), first.partition()));
                 consumeControlFromBufferAsAReplica();
             }
         }
     }
 
-    private void consumeEventsFromBufferAsAReplica() {
+    protected void consumeEventsFromBufferAsAReplica() {
+        if (envConfig.isUnderTest()) {
+            loggerForTest.warn("consumeEventsFromBufferAsAReplica eventsBufferSize:{}", eventsBuffer.size());
+        }
         int index = 0;
         int end = eventsBuffer.size();
         for (ConsumerRecord<String, T> record : eventsBuffer) {
             processEventsAsAReplica(record);
             index++;
-            if (!pollingEvents) {
+            if (polledTopic.equals(PolledTopic.CONTROL)) {
                 if (end > index) {
-                    eventsBuffer = eventsBuffer.subList(index,
-                                                        end);
+                    eventsBuffer = eventsBuffer.subList(index, end);
                 }
                 break;
             }
@@ -420,80 +418,59 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
         }
     }
 
-    private void consumeControlFromBufferAsAReplica() {
-        if (config.isUnderTest()) {
-            loggerForTest.warn("consumeControlFromBufferAsAReplica:{}",
-                               controlBuffer.size());
-        }
-        int index = 0;
-        int end = controlBuffer.size();
+    protected void consumeControlFromBufferAsAReplica() {
         for (ConsumerRecord<String, T> record : controlBuffer) {
             processControlAsAReplica(record);
-            index++;
-            if (!pollingControl) {
-                if (end > index) {
-                    controlBuffer = controlBuffer.subList(index,
-                                                          end);
-                }
-                break;
-            }
-        }
-        if (end == index) {
-            controlBuffer = null;
         }
+        controlBuffer = null;
     }
 
-    private void processEventsAsAReplica(ConsumerRecord<String, T> record) {
-        if (config.isUnderTest()) {
-            loggerForTest.warn("DefaulKafkaConsumer.processEventsAsAReplica record:{}",
-                               record);
-        }
+    protected void processEventsAsAReplica(ConsumerRecord<String, T> record) {
+
+        ItemToProcess item = ItemToProcess.getItemToProcess(record);
         if (record.key().equals(processingKey)) {
-            stopPollingEvents();
-            startPollingControl();
-            stopProcessingNotLeader();
-            consumerHandler.process(ItemToProcess.getItemToProcess(record),
-                                    currentState);
-            saveOffset(record,
-                       kafkaConsumer);
+            lastProcessedEventOffset = record.offset();
+
+            pollControl();
+
             if (logger.isDebugEnabled()) {
-                logger.debug("change topic, switch to consume control");
+                logger.debug("processEventsAsAReplica change topic, switch to consume control, still {} events in the eventsBuffer to consume and processing item:{}.", eventsBuffer.size(), item );
             }
-        } else if (processingReplica) {
-            consumerHandler.process(ItemToProcess.getItemToProcess(record),
-                                    currentState);
-            saveOffset(record,
-                       kafkaConsumer);
+            consumerHandler.process(item, currentState);
+            saveOffset(record, kafkaConsumer);
+
+        } else {
+            if (logger.isDebugEnabled()) {
+                logger.debug("processEventsAsAReplica still {} events in the eventsBuffer to consume and processing item:{}.", eventsBuffer.size(), item );
+            }
+            consumerHandler.process(ItemToProcess.getItemToProcess(record), currentState);
+            saveOffset(record, kafkaConsumer);
         }
     }
 
-    private void processControlAsAReplica(ConsumerRecord<String, T> record) {
-        if (config.isUnderTest()) {
-            loggerForTest.warn("DefaulKafkaConsumer.processControlAsAReplica record:{}",
-                               record);
-        }
+    protected void processControlAsAReplica(ConsumerRecord<String, T> record) {
+
         if (record.offset() == processingKeyOffset + 1 || record.offset() == 0) {
+            lastProcessedControlOffset = record.offset();
             processingKey = record.key();
             processingKeyOffset = record.offset();
             ControlMessage wr = deserialize((byte[]) record.value());
             consumerHandler.processSideEffectsOnReplica(wr.getSideEffects());
-            stopPollingControl();
-            startPollingEvents();
-            startProcessingNotLeader();
-            if (logger.isInfoEnabled()) {
-                logger.info("change topic, switch to consume events");
+
+            pollEvents();
+            if (logger.isDebugEnabled()) {
+                logger.debug("change topic, switch to consume events");
             }
         }
         if (processingKey == null) { // empty topic
             processingKey = record.key();
             processingKeyOffset = record.offset();
         }
-        saveOffset(record,
-                   kafkaSecondaryConsumer);
+        saveOffset(record, kafkaSecondaryConsumer);
     }
 
-    private void saveOffset(ConsumerRecord<String, T> record,
-                            Consumer<String, T> kafkaConsumer) {
+    protected void saveOffset(ConsumerRecord<String, T> record,
+                              Consumer<String, T> kafkaConsumer) {
         Map<TopicPartition, OffsetAndMetadata> map = new HashMap<>();
         map.put(new TopicPartition(record.topic(),
                                    record.partition()),
@@ -501,43 +478,23 @@ public class DefaultKafkaConsumer<T> implements EventConsumer {
         kafkaConsumer.commitSync(map);
     }
 
-    private void stopProcessingNotLeader() {
-        processingReplica = false;
-    }
-
-    private void startProcessingLeader() {
-        processingLeader = true;// the leader starts to process from events topic and publish on control topic
-    }
-
-    private void startPollingControl() {
-        pollingControl = true;
-    }
-
-    private void stopPollingControl() {
-        pollingControl = false;
-    }
-
-    private void startConsume() {
+    protected void startConsume() {
         started = true;
     }
 
-    private void stopConsume() {
+    protected void stopConsume() {
         started = false;
     }
 
-    private void startProcessingNotLeader() {
-        processingReplica = true;
+    protected void pollControl(){
+        polledTopic = PolledTopic.CONTROL;
     }
 
-    private void stopPollingEvents() {
-        pollingEvents = false;
+    protected void pollEvents(){
+        polledTopic = PolledTopic.EVENTS;
     }
 
-    private void stopLeaderProcessing() {
-        processingLeader = false;
-    }
-
-    private void startPollingEvents() {
-        pollingEvents = true;
+    public enum PolledTopic {
+        EVENTS, CONTROL, NONE;
     }
 }
\ No newline at end of file
